@@ -30,15 +30,14 @@
 #include "backends/mixer_backend.h"
 #include "backends/kmix-backends.cpp"
 #include "core/volume.h"
-#include "kmixadaptor.h"
 
 /**
  * Some general design hints. Hierachy is Mixer->MixDevice->Volume
  */
 
 QList<Mixer *> Mixer::s_mixers;
-QString Mixer::_globalMasterCard;
-QString Mixer::_globalMasterCardDevice;
+MasterControl Mixer::_globalMasterCurrent;
+MasterControl Mixer::_globalMasterPreferred;
 
 
 int Mixer::numDrivers()
@@ -65,7 +64,6 @@ QList<Mixer *>& Mixer::mixers()
 Mixer::Mixer( QString& ref_driverName, int device )
     : m_balance(0), _mixerBackend(0L), m_dynamic(false)
 {
-   (void)new KMixAdaptor(this);
     _cardInstance = 0;
     _mixerBackend = 0;
     int driverCount = numDrivers();
@@ -87,10 +85,6 @@ Mixer::Mixer( QString& ref_driverName, int device )
 
 Mixer::~Mixer() {
    // Close the mixer. This might also free memory, depending on the called backend method
-    if ( ! m_dbusName.isEmpty() ) {
-        kDebug(67100) << "Auto-unregistering DBUS object " << m_dbusName;
-    //QDBusConnection::sessionBus().unregisterObject(m_dbusName);
-   }
    close();
    delete _mixerBackend;
 }
@@ -148,8 +142,9 @@ void Mixer::recreateId()
     _id = primaryKeyOfMixer;
 }
 
-
-
+const QString Mixer::dbusPath() {
+    return "/Mixers/" + QString::number( _mixerBackend->m_devnum );
+}
 
 void Mixer::volumeSave( KConfig *config )
 {
@@ -207,28 +202,15 @@ bool Mixer::openIfValid() {
             QString noMaster = "---no-master-detected---";
             setLocalMasterMD(noMaster); // no master
         }
-        connect( _mixerBackend, SIGNAL(controlChanged()), SLOT(controlChangedForwarder()) );
-        connect( _mixerBackend, SIGNAL(controlsReconfigured(const QString&)), SLOT(controlsReconfiguredForwarder(const QString&)) );
-        
-        m_dbusName = "/Mixer" + QString::number(_mixerBackend->m_devnum);
-	kDebug() << "Registering DBUS object " << m_dbusName;
-        bool regResult = QDBusConnection::sessionBus().registerObject(m_dbusName, this);
-	kDebug() << "Registering DBUS object " << m_dbusName << " returns " << regResult;
+        connect( _mixerBackend, SIGNAL(controlChanged()), SIGNAL(controlChanged()) );
+        connect( _mixerBackend, SIGNAL(controlsReconfigured(const QString&)), SIGNAL(controlsReconfigured(const QString&)) );
+    
+        new DBusMixerWrapper(this, dbusPath());
     }
 
     return ok;
 }
 
-void Mixer::controlChangedForwarder()
-{
-    emit controlChanged();
-}
-
-void Mixer::controlsReconfiguredForwarder( const QString& mixer_ID )
-{
-    emit controlsReconfigured(mixer_ID);
-}
-
 /**
  * Closes the mixer.
  * Also, stops the polling timer.
@@ -290,7 +272,9 @@ QString Mixer::translateKernelToWhatsthis(const QString &kernelName)
 /* ------- WRAPPER METHODS. END -------------------------------- */
 
 
-
+int Mixer::balance() const {
+    return m_balance;
+}
 
 void Mixer::setBalance(int balance)
 {
@@ -338,8 +322,11 @@ void Mixer::setBalanceInternal(Volume& vol)
 // should return a name suitable for a human user to read (on a label, ...)
 QString Mixer::readableName()
 {
-  if ( _mixerBackend->m_mixerName.endsWith(":0"))
-     return _mixerBackend->m_mixerName.left(_mixerBackend->m_mixerName.length() - 2);
+  if ( _mixerBackend->m_mixerName.endsWith(":0")) {
+      QString finalName = _mixerBackend->m_mixerName.left(_mixerBackend->m_mixerName.length() - 2);
+      finalName = finalName.append(' ').arg(getCardInstance());
+      return finalName;
+  }
   else
      return _mixerBackend->m_mixerName;
 }
@@ -378,53 +365,74 @@ QString& Mixer::id()
 QString& Mixer::udi(){
     return _mixerBackend->udi();
 }
-void Mixer::setGlobalMaster(QString& ref_card, QString& ref_control)
+
+/**
+ * Set the global master, which is shown in the dock area and which is accesible via the
+ * DBUS masterVolume() method.
+ *
+ * The parameters are taken over as-is, this means without checking for validity.
+ * This allows the User to define a master card that is not always available
+ * (e.g. it is an USB hotplugging device). Also you can set the master at any time you
+ * like, e.g. after reading the KMix configuration file and before actually constructing
+ * the Mixer instances (hint: this method is static!).
+ *
+ * @param ref_card The card id
+ * @param ref_control The control id. The corresponding control must be present in the card.
+ * @param preferred Whether this is the preferred master (auto-selected on coldplug and hotplug).
+ */
+void Mixer::setGlobalMaster(QString ref_card, QString ref_control, bool preferred)
 {
-  // The value is taken over without checking on existence. This allows the User to define
-  // a MasterCard that is not always available (e.g. it is an USB hotplugging device).
-  // Also you can set the master at any time you like, e.g. after reading the KMix configuration file
-  // and before actually constructing the Mixer instances (hint: this mehtod is static!).
-  _globalMasterCard       = ref_card;
-  _globalMasterCardDevice = ref_control;
-  kDebug() << "Mixer::setGlobalMaster() card=" <<ref_card<< " control=" << ref_control;
+    kDebug() << "ref_card=" << ref_card << ", ref_control=" << ref_control << ", preferred=" << preferred;
+    _globalMasterCurrent.set(ref_card, ref_control);
+    if ( preferred )
+        _globalMasterPreferred.set(ref_card, ref_control);
+    kDebug() << "Mixer::setGlobalMaster() card=" <<ref_card<< " control=" << ref_control;
 }
 
 Mixer* Mixer::getGlobalMasterMixerNoFalback()
 {
-   Mixer *mixer = 0;
-   if(Mixer::mixers().count() == 0)
-      return mixer;
-
-   for (int i=0; i< Mixer::mixers().count(); ++i )
+   foreach ( Mixer* mixer, Mixer::mixers())
    {
-      Mixer* mixerTmp = Mixer::mixers()[i];
-      if ( mixerTmp != 0 && mixerTmp->id() == _globalMasterCard ) {
-         //kDebug() << "Mixer::masterCard() found " << _globalMasterCard;
-         mixer = mixerTmp;
-         break;
-      }
+      if ( mixer != 0 && mixer->id() == _globalMasterCurrent.getCard() )
+         return mixer;
    }
-   return mixer;
+   return 0;
 }
 
 Mixer* Mixer::getGlobalMasterMixer()
 {
    Mixer *mixer = getGlobalMasterMixerNoFalback();
    if ( mixer == 0 && Mixer::mixers().count() > 0 ) {
-      // produce fallback
-      mixer = Mixer::mixers()[0];
-      _globalMasterCard = mixer->id();
-      kDebug() << "Mixer::masterCard() fallback to  " << _globalMasterCard;
+      mixer = Mixer::mixers()[0];       // produce fallback
    }
    //kDebug() << "Mixer::masterCard() returns " << mixer->id();
    return mixer;
 }
 
+
+/**
+ * Return the preferred global master.
+ * If there is no preferred global master, returns the current master instead.
+ */
+MasterControl& Mixer::getGlobalMasterPreferred()
+{
+    if ( _globalMasterPreferred.isValid() ) {
+        kDebug() << "Returning preferred master";
+        return _globalMasterPreferred;
+    }
+    else {
+        kDebug() << "Returning current master";
+        return _globalMasterCurrent;
+    }
+}
+
+
 MixDevice* Mixer::getGlobalMasterMD()
 {
    return getGlobalMasterMD(true);
 }
 
+
 MixDevice* Mixer::getGlobalMasterMD(bool fallbackAllowed)
 {
    MixDevice* md = 0;
@@ -437,7 +445,7 @@ MixDevice* Mixer::getGlobalMasterMD(bool fallbackAllowed)
       for(int i=0; i < mixer->_mixerBackend->m_mixDevices.count() ; i++ )
       {
          md = mixer->_mixerBackend->m_mixDevices[i];
-         if ( md->id() == _globalMasterCardDevice ) {
+         if ( md->id() == _globalMasterCurrent.getControl() ) {
             //kDebug() << "Mixer::masterCardDevice() found " << _globalMasterCardDevice;
             break;
          }
@@ -499,22 +507,6 @@ MixDevice* Mixer::getMixdeviceById( const QString& mixdeviceID )
    return md;
 }
 
-// @dcop
-// Used also by the setMasterVolume() method.
-void Mixer::setVolume( const QString& mixdeviceID, int percentage )
-{
-    MixDevice *md = getMixdeviceById( mixdeviceID );
-    if (!md) return;
-
-    Volume& volP = md->playbackVolume();
-    Volume& volC = md->captureVolume();
-
-    // @todo The next call doesn't handle negative volumes correctly.
-    volP.setAllVolumes( (percentage*volP.maxVolume())/100 );
-    volC.setAllVolumes( (percentage*volC.maxVolume())/100 );
-    _mixerBackend->writeVolumeToHW(mixdeviceID, md);
-}
-
 /**
    Call this if you have a *reference* to a Volume object and have modified that locally.
    Pass the MixDevice associated to that Volume to this method for writing back
@@ -539,108 +531,7 @@ void Mixer::commitVolumeChange( MixDevice* md ) {
    }
 }
 
-// @dcop only
-void Mixer::setMasterVolume( int percentage )
-{
-  MixDevice *master = getLocalMasterMD();
-  if (master != 0 ) {
-    setVolume( master->id(), percentage );
-  }
-}
-
-// @dcop
-int Mixer::volume( const QString& mixdeviceID )
-{
-  MixDevice *md= getMixdeviceById( mixdeviceID );
-  if (!md) return 0;
-
-  Volume vol=md->playbackVolume();  // @todo Is hardcoded to PlaybackVolume
-  // @todo This will not work, if minVolume != 0      !!!
-  //       e.g.: minVolume=5 or minVolume=-10
-  // The solution is to check two cases:
-  //     volume < 0 => use minVolume for volumeRange
-  //     volume > 0 => use maxVolume for volumeRange
-  //     If chosen volumeRange==0 => return 0
-  // As this is potentially used often (Sliders, ...), it
-  // should be implemented in the Volume class.
-
-  // For now we go with "maxVolume()", like in the rest of KMix.
-  long volumeRange = vol.maxVolume(); // -vol.minVolume() ;
-  if ( volumeRange == 0 )
-  {
-    return 0;
-  }
-  else
-  {
-     return ( vol.getVolume( Volume::LEFT )*100) / volumeRange ;
-  }
-}
-
-// @dcop , especially for use in KMilo
-void Mixer::setAbsoluteVolume( const QString& mixdeviceID, long absoluteVolume ) {
-    MixDevice *md= getMixdeviceById( mixdeviceID );
-    if (!md) return;
-
-    Volume& volP=md->playbackVolume();
-    Volume& volC=md->captureVolume();
-    volP.setAllVolumes( absoluteVolume );
-    volC.setAllVolumes( absoluteVolume );
-    _mixerBackend->writeVolumeToHW(mixdeviceID, md);
-}
-
-// @dcop , especially for use in KMilo
-long Mixer::absoluteVolume( const QString& mixdeviceID )
-{
-    MixDevice *md = getMixdeviceById( mixdeviceID );
-    if (!md) return 0;
-
-    Volume& volP=md->playbackVolume();  // @todo Is hardcoded to PlaybackVolume
-    long avgVolume=volP.getAvgVolume((Volume::ChannelMask)(Volume::MLEFT | Volume::MRIGHT));
-    return avgVolume;
-}
-
-// @dcop , especially for use in KMilo
-long Mixer::absoluteVolumeMax( const QString& mixdeviceID )
-{
-   MixDevice *md= getMixdeviceById( mixdeviceID );
-   if (!md) return 0;
-
-   Volume vol=md->playbackVolume();  // @todo Is hardcoded to PlaybackVolume
-   long maxVolume=vol.maxVolume();
-   return maxVolume;
-}
-
-// @dcop , especially for use in KMilo
-long Mixer::absoluteVolumeMin( const QString& mixdeviceID )
-{
-   MixDevice *md= getMixdeviceById( mixdeviceID );
-   if (!md) return 0;
-
-   Volume vol=md->playbackVolume();  // @todo Is hardcoded to PlaybackVolume
-   long minVolume=vol.minVolume();
-   return minVolume;
-}
-
-// @dcop
-int Mixer::masterVolume()
-{
-  int vol = 0;
-  MixDevice *master = getLocalMasterMD();
-  if (master != 0 ) {
-    vol = volume( master->id() );
-  }
-  return vol;
-}
-
-// @dbus
-QString Mixer::masterDeviceIndex()
-{
-  MixDevice *master = getLocalMasterMD();
-  return master ? master->id() : QString();
-}
-
-
-// @dcop
+// @dbus, used also in kmix app
 void Mixer::increaseVolume( const QString& mixdeviceID )
 {
     MixDevice *md= getMixdeviceById( mixdeviceID );
@@ -668,7 +559,7 @@ void Mixer::increaseVolume( const QString& mixdeviceID )
   */
 }
 
-// @dcop
+// @dbus
 void Mixer::decreaseVolume( const QString& mixdeviceID )
 {
     MixDevice *md= getMixdeviceById( mixdeviceID );
@@ -699,50 +590,6 @@ void Mixer::decreaseVolume( const QString& mixdeviceID )
     ***********************************************************/
 }
 
-// @dcop
-void Mixer::setMute( const QString& mixdeviceID, bool on )
-{
-  MixDevice *md= getMixdeviceById( mixdeviceID );
-  if (!md) return;
-
-  md->setMuted( on );
-
-     _mixerBackend->writeVolumeToHW(mixdeviceID, md);
-}
-
-// @dcop
-void Mixer::toggleMute( const QString& mixdeviceID )
-{
-    MixDevice *md= getMixdeviceById( mixdeviceID );
-    if (!md) return;
-
-    md->setMuted( ! md->isMuted() );
-    _mixerBackend->writeVolumeToHW(mixdeviceID, md);
-}
-
-// @dcop
-bool Mixer::mute( const QString& mixdeviceID )
-{
-  MixDevice *md= getMixdeviceById( mixdeviceID );
-  if (!md) return true;
-
-  return md->isMuted();
-}
-
-bool Mixer::isRecordSource( const QString& mixdeviceID )
-{
-  MixDevice *md= getMixdeviceById( mixdeviceID );
-  if (!md) return false;
-
-  return md->isRecSource();
-}
-
-/// @DCOP    WHAT DOES THIS METHOD?!?!?
-bool Mixer::isAvailableDevice( const QString& mixdeviceID )
-{
-  return getMixdeviceById( mixdeviceID );
-}
-
 void Mixer::setDynamic ( bool dynamic )
 {
     m_dynamic = dynamic;
